//
// org.osid.OsidObjectForm
//
//     The OSID Java specification for org.osid.OsidObjectForm.
//
// Copyright (C) 2002-2004, 2006-2008 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2009-2012 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid;


/**
 *  <p>The <code> OsidObjectForm </code> is used to create and update <code> 
 *  OsidObjects. </code> The form is not an <code> OsidObject </code> but 
 *  merely a container for data to be sent to an update or create method of a 
 *  session. A provider may or may not combine the <code> OsidObject </code> 
 *  and <code> OsidObjectForm </code> interfaces into a single object. </p> 
 *  
 *  <p> Generally, a set method parallels each get method of an <code> 
 *  OsidObject. </code> Additionally, <code> Metadata </code> may be examined 
 *  for each data element to assist in understanding particular rules 
 *  concerning acceptable data. </p> 
 *  
 *  <p> The form may provide some feedback as to the validity of certain data 
 *  updates before the update transaction is issued to the correspodning 
 *  session but a successful modification of the form is not a guarantee of 
 *  success for the update transaction. A consumer may elect to perform all 
 *  updates within a single update transaction or break up a large update 
 *  intio smaller units. The tradeoff is the granularity of error feedback vs. 
 *  the performance gain of a single transaction. </p> 
 *  
 *  <p> As with all aspects of the OSIDs, nulls cannot be used. Methods to 
 *  clear values are also defined in the form. </p> 
 *  
 *  <p> A new <code> OsidForm </code> should be acquired for each transaction 
 *  upon an <code> OsidObject. </code> Forms should not be reused from one 
 *  object to another even if the supplied data is the same as the forms may 
 *  encapsulate data specific to the object requested. Example of changing a 
 *  display name and a color defined in a color interface extension: </p> 
 *  
 *  <pre>
 *       
 *       
 *       ObjectForm form = session.getObjectFormForUpdate(objectId);
 *       form.setDisplayName("new name");
 *       ColorForm recordForm = form.getFormRecord(colorRecordType);
 *       recordForm.setColor("green");
 *       session.updateObject(objectId, form);
 *       
 *                   
 *       
 *  </pre>
 *  
 */

public interface OsidObjectForm
    extends org.osid.OsidIdentifiableForm,
            org.osid.OsidExtensibleForm,
            org.osid.OsidBrowsableForm {


    /**
     *  Gets the metadata for a display name. 
     *
     *  @return metadata for the display name 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.Metadata getDisplayNameMetadata();


    /**
     *  Sets a display name. A display name is required and if not set, will 
     *  be set by the provider. 
     *
     *  @param  displayName the new display name 
     *  @throws org.osid.InvalidArgumentException <code> displayName </code> 
     *          is invalid 
     *  @throws org.osid.NoAccessException <code> Metadata.isReadonly() 
     *          </code> is <code> true </code> 
     *  @throws org.osid.NullArgumentException <code> displayName </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void setDisplayName(String displayName);


    /**
     *  Clears the display name. 
     *
     *  @throws org.osid.NoAccessException <code> Metadata.isRequired() 
     *          </code> or <code> Metadata.isReadOnly() </code> is <code> true 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearDisplayName();


    /**
     *  Gets the metadata for a description. 
     *
     *  @return metadata for the description 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.Metadata getDescriptionMetadata();


    /**
     *  Sets a description. 
     *
     *  @param  description the new description 
     *  @throws org.osid.InvalidArgumentException <code> description </code> 
     *          is invalid 
     *  @throws org.osid.NoAccessException <code> Metadata.isReadonly() 
     *          </code> is <code> true </code> 
     *  @throws org.osid.NullArgumentException <code> description </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void setDescription(String description);


    /**
     *  Clears the description. 
     *
     *  @throws org.osid.NoAccessException <code> Metadata.isRequired() 
     *          </code> or <code> Metadata.isReadOnly() </code> is <code> true 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearDescription();


    /**
     *  Gets the metadata for a genus type. 
     *
     *  @return metadata for the genus 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.Metadata getGenusTypeMetadata();


    /**
     *  Sets a genus. A genus cannot be cleared because all objects have at 
     *  minimum a root genus. 
     *
     *  @param  genusType the new genus 
     *  @throws org.osid.InvalidArgumentException <code> genusType </code> is 
     *          invalid 
     *  @throws org.osid.NoAccessException <code> Metadata.isReadonly() 
     *          </code> is <code> true </code> 
     *  @throws org.osid.NullArgumentException <code> genusType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void setGenusType(org.osid.type.Type genusType);


    /**
     *  Clears the genus type. 
     *
     *  @throws org.osid.NoAccessException <code> Metadata.isRequired() 
     *          </code> or <code> Metadata.isReadOnly() </code> is <code> true 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearGenusType();
}
