//
// org.osid.OsidObjectQuery
//
//     The OSID Java specification for org.osid.OsidObjectQuery.
//
// Copyright (C) 2002-2004, 2006-2008 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2009-2012 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid;


/**
 *  <p>The <code> OsidObjectQuery </code> is used to assemble search queries. 
 *  An <code> OsidObjectQuery </code> is available from an <code> OsidSession 
 *  </code> and defines methods to query for an <code> OsidObject </code> that 
 *  includes setting a display name and a description. Once the desired 
 *  parameters are set, the <code> OsidQuery </code> is given to the 
 *  designated search method. The same <code> OsidQuery </code> returned from 
 *  the session must be used in the search as the provider may utilize 
 *  implementation-specific data wiithin the object. </p> 
 *  
 *  <p> If multiple data elements are set in this interface, the results 
 *  matching all the given data (eg: AND) are returned. </p> 
 *  
 *  <p> Any match method inside an <code> OsidObjectQuery </code> may be 
 *  invoked multiple times. In the case of a match method, each invocation 
 *  adds an element to an <code> OR </code> expression. Any of these terms may 
 *  also be negated through the <code> match </code> flag. </p> 
 *  
 *  <pre>
 *       
 *       
 *       OsidObjectQuery { OsidQuery.matchDisplayName AND (OsidQuery.matchDescription OR OsidObjectQuery.matchDescription)}
 *       
 *                   
 *       
 *  </pre>
 *  
 *  
 *  <p> <code> OsidObjects </code> allow for the definition of an additonal 
 *  records and the <code> OsidQuery </code> parallels this mechanism. An 
 *  interface type of an <code> OsidObject </code> record must also define the 
 *  corresponding <code> OsidQuery </code> record which is available through 
 *  query interfaces. Multiple requests of these typed interfaces may return 
 *  the same underlying object and thus it is only useful to request once. 
 *  </p> 
 *  
 *  <p> String searches are described using a string search <code> Type 
 *  </code> that indicates the type of regular expression or wildcarding 
 *  encoding. Compatibility with a strings search <code> Type </code> can be 
 *  tested within this interface. </p> 
 *  
 *  <p> As with all aspects of OSIDs, nulls cannot be used. Separate tests are 
 *  available for querying for unset values except for required fields. </p> 
 *  
 *  <p> An example to find all objects whose name starts with "Fred" or whose 
 *  name starts with "Barney", but the word "dinosaur" does not appear in the 
 *  description and not the color is not purple. <code> ColorQuery </code> is 
 *  a record of the object that defines a color. </p> 
 *  
 *  <pre>
 *       
 *       
 *       ObjectObjectQuery query;
 *       
 *                   
 *       
 *  </pre>
 *  
 *  
 *  <pre>
 *       
 *       
 *       query = session.getObjectQuery();
 *       query.matchDisplayName("Fred*", wildcardStringMatchType, true);
 *       query.matchDisplayName("Barney*", wildcardStringMatchType, true);
 *       query.matchDescriptionMatch("dinosaur", wordStringMatchType, false);
 *       
 *       ColorQuery recordQuery;
 *       recordQuery = query.getObjectRecord(colorRecordType);
 *       recordQuery.matchColor("purple", false);
 *       ObjectList list = session.getObjectsByQuery(query);
 *       
 *                   
 *       
 *  </pre>
 *  
 */

public interface OsidObjectQuery
    extends org.osid.OsidIdentifiableQuery,
            org.osid.OsidExtensibleQuery,
            org.osid.OsidBrowsableQuery {


    /**
     *  Adds a display name to match. Multiple display name matches can be 
     *  added to perform a boolean <code> OR </code> among them. 
     *
     *  @param  displayName display name to match 
     *  @param  stringMatchType the string match type 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.InvalidArgumentException <code> displayName </code> 
     *          is not of <code> stringMatchType </code> 
     *  @throws org.osid.NullArgumentException <code> displayName </code> or 
     *          <code> stringMatchType </code> is <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsStringMatchType(stringMatchType) </code> is <code> 
     *          false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchDisplayName(String displayName, 
                                 org.osid.type.Type stringMatchType, 
                                 boolean match);


    /**
     *  Matches any object with a display name. 
     *
     *  @param  match <code> true </code> to match any display name, <code> 
     *          false </code> to match objects with no display name 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyDisplayName(boolean match);


    /**
     *  Clears all display name terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearDisplayNameTerms();


    /**
     *  Adds a description name to match. Multiple description matches can be 
     *  added to perform a boolean <code> OR </code> among them. 
     *
     *  @param  description description to match 
     *  @param  stringMatchType the string match type 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.InvalidArgumentException <code> description </code> 
     *          is not of <code> stringMatchType </code> 
     *  @throws org.osid.NullArgumentException <code> description </code> or 
     *          <code> stringMatchType </code> is <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsStringMatchType(stringMatchType) </code> is <code> 
     *          false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchDescription(String description, 
                                 org.osid.type.Type stringMatchType, 
                                 boolean match);


    /**
     *  Matches a description that has any value. 
     *
     *  @param  match <code> true </code> to match any description, <code> 
     *          false </code> to match descriptions with no values 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyDescription(boolean match);


    /**
     *  Clears all description terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearDescriptionTerms();


    /**
     *  Sets a <code> Type </code> for querying objects of a given genus. A 
     *  genus type matches if the specified type is the same genus as the 
     *  object genus type. 
     *
     *  @param  genusType the object genus type 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> genusType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchGenusType(org.osid.type.Type genusType, boolean match);


    /**
     *  Matches an object that has any genus type. 
     *
     *  @param  match <code> true </code> to match any genus type, <code> 
     *          false </code> to match objects with no genus type 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyGenusType(boolean match);


    /**
     *  Clears all genus type terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearGenusTypeTerms();


    /**
     *  Sets a <code> Type </code> for querying objects of a given genus. A 
     *  genus type matches if the specified type is the same genus as the 
     *  object or if the specified type is an ancestor of the object genus in 
     *  a type hierarchy. 
     *
     *  @param  genusType the object genus type 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> genusType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchParentGenusType(org.osid.type.Type genusType, 
                                     boolean match);


    /**
     *  Clears all genus type terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearParentGenusTypeTerms();


    /**
     *  Matches an object with a relationship to the given subject. 
     *
     *  @param  subjectId a subject <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> subjectId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchSubjectId(org.osid.id.Id subjectId, boolean match);


    /**
     *  Clears all subject <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSubjectIdTerms();


    /**
     *  Tests if a <code> SubjectQuery </code> is available. 
     *
     *  @return <code> true </code> if a subject query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsSubjectQuery();


    /**
     *  Gets the query for a subject. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the subject query 
     *  @throws org.osid.UnimplementedException <code> supportsSubjectQuery() 
     *          </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsSubjectQuery() </code> is <code> true. </code> 
     */

    public org.osid.ontology.SubjectQuery getSubjectQuery();


    /**
     *  Matches an object that has any relationship to a <code> Subject. 
     *  </code> 
     *
     *  @param  match <code> true </code> to match any subject, <code> false 
     *          </code> to match objects with no subjects 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnySubject(boolean match);


    /**
     *  Clears all subject terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSubjectTerms();


    /**
     *  Tests if a <code> RelevancyQuery </code> is available to provide 
     *  queries about the relationships to <code> Subjects. </code> 
     *
     *  @return <code> true </code> if a relevancy entry query is available, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsSubjectRelevancyQuery();


    /**
     *  Gets the query for a subject relevancy. Multiple retrievals produce a 
     *  nested <code> OR </code> term. 
     *
     *  @return the relevancy query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsSubjectRelevancyQuery() </code> is <code> false 
     *          </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsSubjectRelevancyQuery() </code> is <code> true. 
     *              </code> 
     */

    public org.osid.ontology.RelevancyQuery getSubjectRelevancyQuery();


    /**
     *  Clears all subject relevancy terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSubjectRelevancyTerms();


    /**
     *  Matches an object mapped to the given state. 
     *
     *  @param  stateId a state <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> stateId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchStateId(org.osid.id.Id stateId, boolean match);


    /**
     *  Clears all state <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearStateIdTerms();


    /**
     *  Tests if a <code> StateQuery </code> is available to provide queries 
     *  of processed objects. 
     *
     *  @return <code> true </code> if a state query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsStateQuery();


    /**
     *  Gets the query for a state. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the journal entry query 
     *  @throws org.osid.UnimplementedException <code> supportsStateQuery() 
     *          </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsStateQuery() </code> is <code> true. </code> 
     */

    public org.osid.process.StateQuery getStateQuery();


    /**
     *  Matches an object that has any mapping to a <code> State </code> in 
     *  the given <code> Process. </code> 
     *
     *  @param  match <code> true </code> to match any state, <code> false 
     *          </code> to match objects with no states 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyState(boolean match);


    /**
     *  Clears all state terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearStateTerms();


    /**
     *  Matches an object that has the given comment. 
     *
     *  @param  commentId a comment <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> commentId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchCommentId(org.osid.id.Id commentId, boolean match);


    /**
     *  Clears all comment <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCommentIdTerms();


    /**
     *  Tests if a <code> CommentQuery </code> is available. 
     *
     *  @return <code> true </code> if a comment query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsCommentQuery();


    /**
     *  Gets the query for a comment. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the comment query 
     *  @throws org.osid.UnimplementedException <code> supportsCommentQuery() 
     *          </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsCommentQuery() </code> is <code> true. </code> 
     */

    public org.osid.commenting.CommentQuery getCommentQuery();


    /**
     *  Matches an object that has any <code> Comment </code> in the given 
     *  <code> Book. </code> 
     *
     *  @param  match <code> true </code> to match any comment, <code> false 
     *          </code> to match objects with no comments 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyComment(boolean match);


    /**
     *  Clears all comment terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCommentTerms();


    /**
     *  Matches an object that has the given journal entry. 
     *
     *  @param  journalEntryId a journal entry <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> journalEntryId </code> 
     *          is <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchJournalEntryId(org.osid.id.Id journalEntryId, 
                                    boolean match);


    /**
     *  Clears all journal entry <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearJournalEntryIdTerms();


    /**
     *  Tests if a <code> JournalEntry </code> is available to provide queries 
     *  of journaled <code> OsidObjects. </code> 
     *
     *  @return <code> true </code> if a journal entry query is available, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsJournalEntryQuery();


    /**
     *  Gets the query for a journal entry. Multiple retrievals produce a 
     *  nested <code> OR </code> term. 
     *
     *  @return the journal entry query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsJournalEntryQuery() </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsJournalEntryQuery() </code> is <code> true. 
     *              </code> 
     */

    public org.osid.journaling.JournalEntryQuery getJournalEntryQuery();


    /**
     *  Matches an object that has any <code> JournalEntry </code> in the 
     *  given <code> Journal. </code> 
     *
     *  @param  match <code> true </code> to match any journal entry, <code> 
     *          false </code> to match objects with no journal entries 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyJournalEntry(boolean match);


    /**
     *  Clears all journal entry terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearJournalEntryTerms();


    /**
     *  Tests if a <code> StatisticQuery </code> is available to provide 
     *  statistical queries. 
     *
     *  @return <code> true </code> if a statistic query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsStatisticQuery();


    /**
     *  Gets the query for a statistic. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the statistic query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsStatisticQuery() </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsStatisticQuery() </code> is <code> true. </code> 
     */

    public org.osid.metering.StatisticQuery getStatisticQuery();


    /**
     *  Matches an object that has any <code> Statistic </code> . 
     *
     *  @param  match <code> true </code> to match any statistic, <code> false 
     *          </code> to match objects with no statistics 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyStatistic(boolean match);


    /**
     *  Clears all statistic terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearStatisticTerms();


    /**
     *  Matches an object that has the given credit. 
     *
     *  @param  creditId a credit <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> creditId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchCreditId(org.osid.id.Id creditId, boolean match);


    /**
     *  Clears all credit <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCreditIdTerms();


    /**
     *  Tests if a <code> CreditQuery </code> is available to provide queries 
     *  of related acknowledgements. 
     *
     *  @return <code> true </code> if a credit query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsCreditQuery();


    /**
     *  Gets the query for an ackowledgement credit. Multiple retrievals 
     *  produce a nested <code> OR </code> term. 
     *
     *  @return the credit query 
     *  @throws org.osid.UnimplementedException <code> supportsCreditQuery() 
     *          </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsCreditQuery() </code> is <code> true. </code> 
     */

    public org.osid.acknowledgement.CreditQuery getCreditQuery();


    /**
     *  Matches an object that has any <code> Credit. </code> 
     *
     *  @param  match <code> true </code> to match any credit, <code> false 
     *          </code> to match objects with no credits 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyCredit(boolean match);


    /**
     *  Clears all credit terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCreditTerms();


    /**
     *  Matches an object that has the given relationship. 
     *
     *  @param  relationshipId a relationship <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> relationshipId </code> 
     *          is <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchRelationshipId(org.osid.id.Id relationshipId, 
                                    boolean match);


    /**
     *  Clears all relationship <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearRelationshipIdTerms();


    /**
     *  Tests if a <code> RelationshipQuery </code> is available. 
     *
     *  @return <code> true </code> if a relationship query is available, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsRelationshipQuery();


    /**
     *  Gets the query for relationship. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the relationship query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsRelationshipQuery() </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsRelationshipQuery() </code> is <code> true. 
     *              </code> 
     */

    public org.osid.relationship.RelationshipQuery getRelationshipQuery();


    /**
     *  Matches an object that has any <code> Relationship. </code> 
     *
     *  @param  match <code> true </code> to match any relationship, <code> 
     *          false </code> to match objects with no relationships 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyRelationship(boolean match);


    /**
     *  Clears all relationship terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearRelationshipTerms();


    /**
     *  Matches an object that has a relationship to the given peer <code> Id. 
     *  </code> 
     *
     *  @param  peerId a relationship peer <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> peerId </code> is <code> 
     *          null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchRelationshipPeerId(org.osid.id.Id peerId, boolean match);


    /**
     *  Clears all relationship <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearRelationshipPeerIdTerms();
}
