//
// org.osid.OsidRelationship
//
//     The OSID Java specification for org.osid.OsidRelationship.
//
// Copyright (C) 2002-2004, 2006-2008 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2009-2012 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid;


/**
 *  <p>A <code> Relationship </code> associates two OSID objects. 
 *  Relationships are transient. They define a date range for which they are 
 *  in effect. </p> 
 *  
 *  <p> Unlike other <code> OsidObjects </code> that rely on the auxiliary 
 *  Journaling OSID to track variance over time, <code> OsidRelationships 
 *  </code> introduce a different concept of time independent from journaling. 
 *  For example, in the present, a student was registered in a course and 
 *  dropped it. The relationship between the student and the course remains 
 *  pertinent, independent of any journaled changes that may have occurred to 
 *  either the student or the course. </p> 
 *  
 *  <p> Once the student has dropped the course, the relationship has expired 
 *  such that <code> isEffective() </code> becomes false. It can be inferred 
 *  that during the period of the effective dates, the student was actively 
 *  registered in the course. Here is an example: </p> 
 *  
 *  <ul>
 *      <li> T1. September 1: Student registers for course for grades </li> 
 *      <li> T2. September 10: Student drops course </li> 
 *      <li> T3. September 15: Student re-registers for course pass/fail </li> 
 *  </ul>
 *  
 *  
 *  <p> The relationships are: </p> 
 *  
 *  <pre>
 *       
 *       
 *       T1. R1 {effective,   September 1  -&gt; end of term,  data=grades}
 *       T2. R1 {ineffective, September 1  -&gt; September 10, data=grades}
 *       T3. R1 {ineffective, September 1  -&gt; September 10, data=grades}
 *           R2 {effective,   September 10 -&gt; end of term,  data=p/f}
 *       
 *                   
 *       
 *  </pre>
 *  
 *  
 *  <p> An OSID Provider may also permit dates to be set in the future in 
 *  which case the relationship can become automatically become effective at a 
 *  future time and later expire. More complex effectiveness management can be 
 *  done through other rule-based services. </p> 
 *  
 *  <p> OSID Consumer lookups and queries of relationships need to consider 
 *  that it may be only effective relationshps are of interest. </p>
 */

public interface OsidRelationship
    extends org.osid.OsidObject,
            org.osid.Temporal {


    /**
     *  Tests if a reason this relationship came to an end is known. 
     *
     *  @return <code> true </code> if an end reason is available, <code> 
     *          false </code> otherwise 
     *  @throws org.osid.IllegalStateException <code> isEffective() </code> is 
     *          <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasEndReason();


    /**
     *  Gets a state <code> Id </code> indicating why this relationship has 
     *  ended. 
     *
     *  @return a state <code> Id </code> 
     *  @throws org.osid.IllegalStateException <code> hasEndReason() </code> 
     *          is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getEndReasonId();


    /**
     *  Gets a state indicating why this relationship has ended. 
     *
     *  @return a state 
     *  @throws org.osid.IllegalStateException <code> hasEndReason() </code> 
     *          is <code> false </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.process.State getEndReason()
        throws org.osid.OperationFailedException;
}
