/**
 * <p>The Open Service Interface Definitions for the org.osid.authorization
 * service.</p>
 * 
 * <p>The Authorization OSID manages and queries authorizations. </p> 
 * <h4> Authorizations </h4> 
 * 
 * <p> An <code> Authorization </code> is an <code> OsidRelationship </code> 
 * that defines who can do what to what. The grammar of an authorization 
 * incluides the subject or the actor (who), the action or verb (do what), and 
 * the object or context (to what). All three of these components must exist 
 * in an authorization for it to have any explicit meaning. An <code> 
 * Authorization </code> is a mapping among these three components. </p> 
 * 
 * <ul>
 *     <li> <code> Agent: </code> the actor (eg: tom@coppeto.org) </li> 
 *     <li> <code> Function: </code> the action (eg: create purchase order) 
 *     </li> 
 *     <li> <code> Qualifier: </code> the object or context within a Function 
 *     (eg: on account 1967) </li> 
 * </ul>
 * 
 * 
 * <p> This tuple in essence defines a role. "Instructor" is not a role and is 
 * not suitable for making an authorization decision. "Instructs Physics 101", 
 * both the function and qualifier, defines the complete role (within the 
 * context of a particular college) that can be used for an authorization 
 * decision. </p> 
 * 
 * <p> The basic service of the Authorization OSID is to provide a means for 
 * asking whether a given <code> Agent </code> is authorized to perform a 
 * <code> Function </code> with a <code> Qualifier, </code> in other words, if 
 * such a mapping exists. The Agent will generally be obtained from an 
 * Authentication service and the <code> Function </code> and <code> Qualifier 
 * </code> generally known to the consuming application (a server process 
 * needing to protect some resource). </p> 
 * <h4> Example </h4> 
 * 
 * <pre>
 *      
 *      
 *      Authentication auth = authNValidationSession.authenticate(creds);
 *      
 *      AuthorizationSession session = authZManager.getAuthorizationSession();
 *      boolean authorized = session.isAuthorized(auth.getAgentId(), functionId, qualifierId);
 *      
 *              
 *      
 * </pre>
 * 
 * 
 * <p> The rest of the Authorization OSID is concerned with managing 
 * authorizations. </p> 
 * <h4> Explicit/Implicit Authorizations </h4> 
 * 
 * <p> Authorizations can be explcit or implcit. Explicit authorizations are 
 * managed while implcit authorizations are derived from <code> Resources, 
 * </code> <code> Function </code> and <code> Qualifier </code> hierrachies. 
 * Examples of implcit authorizations: </p> 
 * 
 * <ul>
 *     <li> The Authorization OSID can accept a <code> Resource </code> in 
 *     lieu of an <code> Agent </code> as the actor so a Person, Group or 
 *     Organization may be used to specify an authorization. In this case, the 
 *     explicit authorization is the one containing the <code> Resource 
 *     </code> and an implicit authorization exists for each <code> Agent. 
 *     </code> </li> 
 *     <li> <code> Qualifiers </code> only exist as Hierarchy Nodes since the 
 *     Authorization OSID does not manage the objects used as qualifiers but 
 *     may manage directly, or have access to, a Hierarchy service to obtain 
 *     the identity and relationship among these objects. An explicit 
 *     authorization for a given <code> Qualifier </code> creates an implcit 
 *     authorization for every child of that <code> Qualifier. </code> </li> 
 * </ul>
 * 
 * 
 * <p> The Authorization OSID manages <code> Functions </code> directly 
 * through its owned defined sessions and exposes actors via the Resource 
 * OSID. <code> Qualifiers </code> are only exposed through the Hierarchy 
 * service as the Authorization service doesn't have anything to say about the 
 * objects represented by the <code> Qualifiers. </code> </p> 
 * <h4> Vault Cataloging </h4> 
 * 
 * <p> <code> Authorizations, Functions </code> and <code> Qualifiers </code> 
 * may be organized into one or many <code> Vaults. </code> This serves to 
 * categorize authorizatiion data for the purpose of browsing or auditing. 
 * <code> Vaults </code> are hierarchical where each node includes all the 
 * authorization data of its children. A single root node will make available 
 * all known authorizations and is a reasonable choice for a default <code> 
 * Vault </code> for a non-federated aware consumer. A federated authorization 
 * scheme is one in which <code> Vaults </code> are available for selection. 
 * </p> 
 * <h4> Notifications </h4> 
 * 
 * <p> Certain consumers may wish to be notified of changes within the 
 * service. Authorization supports notifications via <code> 
 * AuthorizatioNotificationSession, </code> <code> FunctionNotificationSession 
 * </code> and <code> VaultNotificationSession. </code> </p> 
 * 
 * <pre>
 *      
 *      
 *      if (manager.supportsAuthorizationNotification()) {
 *          AuthorizationNotificationSession ans = manager.getAuthorizationNotificationSession(receiver);           
 *          ans.registerForDeletedAuthorizations();
 *      }
 *      
 *      AuthorizationReceiver receiver {
 *          newAuthorization(Authorization a) {print("authorization created");}
 *          deletedAuthorization(Authorization a) {print("authorization removed");}
 *      }
 *      
 *              
 *      
 * </pre>
 * 
 * <h4> Sub Packages </h4> 
 * 
 * <p> The Authorization OSID includes an Authorization Rules OSID for 
 * managing the effectiveness of <code> Authorizations. </code> </p>
 * 
 * <br/><hr/><br/>
 * 
 * <p>Copyright &copy; 2002-2004, 2007-2008 Massachusetts Institute of 
 * Technology. </p> 
 * 
 * <p> Copyright &copy; 2009-2010 Ingenescus. All Rights Reserved. </p>
 * 
 * <p>This Work is being provided by the copyright holder(s) subject to the 
 * following license. By obtaining, using and/or copying this Work, you agree 
 * that you have read, understand, and will comply with the following terms 
 * and conditions. </p> 
 * 
 * <p> Permission to use, copy and distribute unmodified versions of this 
 * Work, for any purpose, without fee or royalty is hereby granted, provided 
 * that you include the above copyright notices and the terms of this license 
 * on ALL copies of the Work or portions thereof. </p> 
 * 
 * <p> You may modify or create Derivatives of this Work only for your 
 * internal purposes. You shall not distribute or transfer any such Derivative 
 * of this Work to any location or to any third party. For the purposes of 
 * this license, "Derivative" shall mean any derivative of the Work as defined 
 * in the United States Copyright Act of 1976, such as a translation or 
 * modification. </p> 
 * 
 * <p> This Work and the information contained herein is provided on an "AS 
 * IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
 * NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN 
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 * CONNECTION WITH THE WORK OR THE USE OR OTHER DEALINGS IN THE WORK. </p> 
 * 
 * <p> The export of software employing encryption technology may require a 
 * specific license from the United States Government. It is the 
 * responsibility of any person or organization contemplating export to obtain 
 * such a license before exporting this Work. </p>
 */

package org.osid.authorization;
