//
// org.osid.billing.EntryLookupSession
//
//     The OSID Java specification for org.osid.billing.EntryLookupSession.
//
// Copyright (C) 2012 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.billing;


/**
 *  <p>This session provides methods for retrieving <code> Entry </code> 
 *  objects. </p> 
 *  
 *  <p> This session defines views that offer differing behaviors when 
 *  retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete set or is an error condition 
 *      </li> 
 *      <li> isolated business view: All entry methods in this session 
 *      operate, retrieve and pertain to entries defined explicitly in the 
 *      current business. Using an isolated view is useful for managing <code> 
 *      Entries </code> with the <code> EntryAdminSession. </code> </li> 
 *      <li> federated business view: All entry methods in this session 
 *      operate, retrieve and pertain to all entries defined in this business 
 *      and any other entries implicitly available in this business through 
 *      business inheritence. </li> 
 *      <li> effective entry view: All entry lookup methods return entries 
 *      where the current date falls in between the effective dates inclusive. 
 *      </li> 
 *      <li> any effective entry view: Entries of any effective date are 
 *      returned from methods. </li> 
 *  </ul>
 *  
 *  
 *  <p> Generally, the comparative view should be used for most applications 
 *  as it permits operation even if there is data that cannot be accessed. The 
 *  methods <code> useFederatedBusinessView() </code> and <code> 
 *  useIsolatedBusinessView() </code> behave as a radio group and one should 
 *  be selected before invoking any lookup methods. </p>
 */

public interface EntryLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Business </code> <code> Id </code> associated with 
     *  this session. 
     *
     *  @return the <code> Business Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getBusinessId();


    /**
     *  Gets the <code> Business </code> associated with this session. 
     *
     *  @return the business 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.Business getBusiness()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Entry </code> lookups. A return 
     *  of true does not guarantee successful authorization. A return of false 
     *  indicates that it is known all methods in this session will result in 
     *  a <code> PERMISSION_DENIED. </code> This is intended as a hint to an 
     *  application that may opt not to offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupEntries();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeEntryView();


    /**
     *  A complete view of the <code> Entry </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryEntryView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include entries in businesses which are children of this business in 
     *  the business hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedBusinessView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this business only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedBusinessView();


    /**
     *  Only entries whose effective dates are current are returned by methods 
     *  in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveEntryView();


    /**
     *  All entries of any effective dates are returned by methods in this 
     *  session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveEntryView();


    /**
     *  Gets the <code> Entry </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Entry </code> may have a different <code> Id </code> than requested, 
     *  such as the case where a duplicate <code> Id </code> was assigned to 
     *  an <code> Entry </code> and retained for compatibility. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  entryId <code> Id </code> of the <code> Entry </code> 
     *  @return the entry 
     *  @throws org.osid.NotFoundException <code> entryId </code> not found 
     *  @throws org.osid.NullArgumentException <code> entryId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method is must be implemented. 
     */

    public org.osid.billing.Entry getEntry(org.osid.id.Id entryId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  entries specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Entries </code> may be omitted from the list and 
     *  may present the elements in any order including returning a unique 
     *  set. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  entryIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> entryIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByIds(org.osid.id.IdList entryIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> corresponding to the given entry 
     *  genus <code> Type </code> which does not include entries of genus 
     *  types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  entryGenusType an entry genus type 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> entryGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByGenusType(org.osid.type.Type entryGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> corresponding to the given entry 
     *  genus <code> Type </code> and include any additional entries with 
     *  genus types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  entryGenusType an entry genus type 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> entryGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByParentGenusType(org.osid.type.Type entryGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> containing the given entry record 
     *  <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  entryRecordType an entry record type 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> entryRecordType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByRecordType(org.osid.type.Type entryRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> in the given billing period. <code> 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  periodId a billing period <code> Id </code> 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> periodId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByPeriod(org.osid.id.Id periodId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> effective during the entire given 
     *  date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective 
     *  in addition to being effective in the given date range. In any 
     *  effective mode, effective entries and those currently expired are 
     *  returned. 
     *
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from or to </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesOnDate(org.osid.calendaring.DateTime from, 
                                                       org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given customer. <code> 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> customerId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesForCustomer(org.osid.id.Id customerId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given customer and effective 
     *  during the entire given date range inclusive but not confined to the 
     *  date range. 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective 
     *  in addition to being effective in the given date range. In any 
     *  effective mode, effective entries and those currently expired are 
     *  returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> customerId, from, or to 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesForCustomerOnDate(org.osid.id.Id customerId, 
                                                                  org.osid.calendaring.DateTime from, 
                                                                  org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given customer in a billing 
     *  period. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @param  periodId a billing period <code> Id </code> 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> customerId </code> or 
     *          <code> periodId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByPeriodForCustomer(org.osid.id.Id customerId, 
                                                                    org.osid.id.Id periodId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> in the given billing period for the 
     *  given customer and effective during the entire given date range 
     *  inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective 
     *  in addition to being effective in the given date range. In any 
     *  effective mode, effective entries and those currently expired are 
     *  returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @param  periodId a billing period <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> customerId, periodId, 
     *          from, or to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByPeriodForCustomerOnDate(org.osid.id.Id customerId, 
                                                                          org.osid.id.Id periodId, 
                                                                          org.osid.calendaring.DateTime from, 
                                                                          org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given item. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  itemId an item <code> Id </code> 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> itemId </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesForItem(org.osid.id.Id itemId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given item and effective 
     *  during the entire given date range inclusive but not confined to the 
     *  date range. 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective 
     *  in addition to being effective in the given date range. In any 
     *  effective mode, effective entries and those currently expired are 
     *  returned. 
     *
     *  @param  itemId an item <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> itemId, from, or to 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesForItemOnDate(org.osid.id.Id itemId, 
                                                              org.osid.calendaring.DateTime from, 
                                                              org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given item in a billing 
     *  period. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  itemId an item <code> Id </code> 
     *  @param  periodId a billing period <code> Id </code> 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> itemId </code> or <code> 
     *          periodId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByPeriodForItem(org.osid.id.Id itemId, 
                                                                org.osid.id.Id periodId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> in the given billing period for the 
     *  given item and effective during the entire given date range inclusive 
     *  but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective 
     *  in addition to being effective in the given date range. In any 
     *  effective mode, effective entries and those currently expired are 
     *  returned. 
     *
     *  @param  itemId an item <code> Id </code> 
     *  @param  periodId a billing period <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> itemId, periodId, from, 
     *          or to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByPeriodForItemOnDate(org.osid.id.Id itemId, 
                                                                      org.osid.id.Id periodId, 
                                                                      org.osid.calendaring.DateTime from, 
                                                                      org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given customer and item. 
     *  <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @param  itemId an item <code> Id </code> 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> customerId </code> or 
     *          <code> itemId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesForCustomerAndItem(org.osid.id.Id customerId, 
                                                                   org.osid.id.Id itemId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given customer and item and 
     *  effective during the entire given date range inclusive but not 
     *  confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective 
     *  in addition to being effective in the given date range. In any 
     *  effective mode, effective entries and those currently expired are 
     *  returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @param  itemId an item <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> customerId, itemId, 
     *          from, or to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesForCustomerAndItemOnDate(org.osid.id.Id customerId, 
                                                                         org.osid.id.Id itemId, 
                                                                         org.osid.calendaring.DateTime from, 
                                                                         org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given customer and item in a 
     *  billing period. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  entries or an error results. Otherwise, the returned list may contain 
     *  only those entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @param  itemId an item <code> Id </code> 
     *  @param  periodId a period <code> Id </code> 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.NullArgumentException <code> customerId, itemId, 
     *          </code> or <code> periodId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByPeriodForCustomerAndItem(org.osid.id.Id customerId, 
                                                                           org.osid.id.Id itemId, 
                                                                           org.osid.id.Id periodId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EntryList </code> for the given customer and item in a 
     *  billing period and effective during the entire given date range 
     *  inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective 
     *  in addition to being effective in the given date range. In any 
     *  effective mode, effective entries and those currently expired are 
     *  returned. 
     *
     *  @param  customerId a customer <code> Id </code> 
     *  @param  itemId an item <code> Id </code> 
     *  @param  periodId a period <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Entry </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> customerId, itemId, 
     *          periodId, from, or to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntriesByPeriodForCustomerAndItemOnDate(org.osid.id.Id customerId, 
                                                                                 org.osid.id.Id itemId, 
                                                                                 org.osid.id.Id periodId, 
                                                                                 org.osid.calendaring.DateTime from, 
                                                                                 org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Entries. </code> 
     *  
     *  In plenary mode, the returned list contains all known entries or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  entries that are accessible through this session. 
     *  
     *  In effective mode, entries are returned that are currently effective. 
     *  In any effective mode, effective entries and those currently expired 
     *  are returned. 
     *
     *  @return a list of <code> Entries </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.EntryList getEntries()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
