//
// org.osid.calendaring.EventQuery
//
//     The OSID Java specification for org.osid.calendaring.EventQuery.
//
// Copyright (C) 2002-2004 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2008-2011 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.calendaring;


/**
 *  <p>This is the query for searching events. Each method match request 
 *  produces an <code> AND </code> term while multiple invocations of a method 
 *  produces a nested <code> OR. </code> </p>
 */

public interface EventQuery
    extends org.osid.OsidObjectQuery,
            org.osid.OsidTemporalQuery,
            org.osid.OsidContainableQuery {


    /**
     *  Matches an event that is implicitly generated. 
     *
     *  @param  match <code> true </code> to match events implicitly 
     *          generated, <code> false </code> to match events explicitly 
     *          defined 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchImplicit(boolean match);


    /**
     *  Clears the implcit terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearImplicitTerms();


    /**
     *  Matches the event duration between the given range inclusive. 
     *
     *  @param  low low duration range 
     *  @param  high high duration range 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.InvalidArgumentException <code> high </code> is less 
     *          than <code> low </code> 
     *  @throws org.osid.NullArgumentException <code> high </code> or <code> 
     *          low </code> is <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchDuration(org.osid.calendaring.Duration low, 
                              org.osid.calendaring.Duration high, 
                              boolean match);


    /**
     *  Matches an event that has any duration. 
     *
     *  @param  match <code> true </code> to match events with any duration, 
     *          <code> false </code> to match events with no start time 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyDuration(boolean match);


    /**
     *  Clears the duration terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearDurationTerms();


    /**
     *  Matches events that related to the recurring event. 
     *
     *  @param  recurringEventId an <code> Id </code> for a recurring event 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> recurringEventId </code> 
     *          is <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchRecurringEventId(org.osid.id.Id recurringEventId, 
                                      boolean match);


    /**
     *  Clears the recurring event terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearRecurringEventIdTerms();


    /**
     *  Tests if a <code> RecurringEventQuery </code> is available for 
     *  querying recurring events. 
     *
     *  @return <code> true </code> if a recurring event query is available, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsRecurringEventQuery();


    /**
     *  Gets the query for a recurring event. Multiple retrievals produce a 
     *  nested <code> OR </code> term. 
     *
     *  @return the recurring event query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsRecurringEventQuery() </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsRecurringEventQuery() </code> is <code> true. 
     *              </code> 
     */

    public org.osid.calendaring.RecurringEventQuery getRecurringEventQuery();


    /**
     *  Matches an event that is part of any recurring event. 
     *
     *  @param  match <code> true </code> to match events part of any 
     *          recurring event, <code> false </code> to match only standalone 
     *          events 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyRecurringEvent(boolean match);


    /**
     *  Clears the recurring event terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearRecurringEventTerms();


    /**
     *  Matches events that relate to the superseding event. 
     *
     *  @param  supersedingEventId an <code> Id </code> for a superseding 
     *          event 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchSupersedingEventId(org.osid.id.Id supersedingEventId, 
                                        boolean match);


    /**
     *  Clears the superseding events type terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSupersedingEventIdTerms();


    /**
     *  Tests if a <code> SupersedingEventQuery </code> is available for 
     *  querying offset events. 
     *
     *  @return <code> true </code> if a superseding event query is available, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsSupersedingEventQuery();


    /**
     *  Gets the query for a superseding event. Multiple retrievals produce a 
     *  nested <code> OR </code> term. 
     *
     *  @return the superseding event query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsSupersedingEventQuery() </code> is <code> false 
     *          </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsSupersedingEventQuery() </code> is <code> true. 
     *              </code> 
     */

    public org.osid.calendaring.SupersedingEventQuery getSupersedingEventQuery();


    /**
     *  Matches any superseding event. 
     *
     *  @param  match <code> true </code> to match any superseding events, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnySupersedingEvent(boolean match);


    /**
     *  Clears the superseding event terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSupersedingEventTerms();


    /**
     *  Matches events that relates to the offset event. 
     *
     *  @param  offsetEventId an <code> Id </code> for an offset event 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchOffsetEventId(org.osid.id.Id offsetEventId, boolean match);


    /**
     *  Clears the recurring events type terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearOffsetEventIdTerms();


    /**
     *  Tests if an <code> OffsetEventQuery </code> is available for querying 
     *  offset events. 
     *
     *  @return <code> true </code> if an offset event query is available, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsOffsetEventQuery();


    /**
     *  Gets the query for an offset event. Multiple retrievals produce a 
     *  nested <code> OR </code> term. 
     *
     *  @return the offset event query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsOffsetEventQuery() </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsOffsetEventQuery() </code> is <code> true. </code> 
     */

    public org.osid.calendaring.OffsetEventQuery getOffsetEventQuery();


    /**
     *  Matches any offset event. 
     *
     *  @param  match <code> true </code> to match any offset events, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyOffsetEvent(boolean match);


    /**
     *  Clears the offset event terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearOffsetEventTerms();


    /**
     *  Matches the location description string. 
     *
     *  @param  location location string 
     *  @param  stringMatchType string match type 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.InvalidArgumentException <code> location </code> is 
     *          not of <code> stringMatchType </code> 
     *  @throws org.osid.NullArgumentException <code> location </code> or 
     *          <code> stringMatchType </code> is <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsStringMatchType(stringMatchType) </code> is <code> 
     *          false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchLocationDescription(String location, 
                                         org.osid.type.Type stringMatchType, 
                                         boolean match);


    /**
     *  Matches an event that has any location description assigned. 
     *
     *  @param  match <code> true </code> to match events with any location 
     *          description, <code> false </code> to match events with no 
     *          location description 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyLocationDescription(boolean match);


    /**
     *  Clears the location description terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearLocationDescriptionTerms();


    /**
     *  Sets the location <code> Id </code> for this query. 
     *
     *  @param  locationId a location <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> locationId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchLocationId(org.osid.id.Id locationId, boolean match);


    /**
     *  Clears the location <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearLocationIdTerms();


    /**
     *  Tests if a <code> LocationQuery </code> is available for querying 
     *  locations. 
     *
     *  @return <code> true </code> if a location query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsLocationQuery();


    /**
     *  Gets the query for a location. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the location query 
     *  @throws org.osid.UnimplementedException <code> supportsLocationQuery() 
     *          </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsLocationQuery() </code> is <code> true. </code> 
     */

    public org.osid.mapping.LocationQuery getLocationQuery();


    /**
     *  Matches an event that has any location assigned. 
     *
     *  @param  match <code> true </code> to match events with any location, 
     *          <code> false </code> to match events with no location 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyLocation(boolean match);


    /**
     *  Clears the location terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearLocationTerms();


    /**
     *  Sets the sponsor <code> Id </code> for this query. 
     *
     *  @param  sponsorId a sponsor <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> sponsorId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchSponsorId(org.osid.id.Id sponsorId, boolean match);


    /**
     *  Clears the sponsor <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSponsorIdTerms();


    /**
     *  Tests if a <code> LocationQuery </code> is available for querying 
     *  sponsors. 
     *
     *  @return <code> true </code> if a sponsor query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsSponsorQuery();


    /**
     *  Gets the query for a sponsor. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the sponsor query 
     *  @throws org.osid.UnimplementedException <code> supportsSponsorQuery() 
     *          </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsSponsorQuery() </code> is <code> true. </code> 
     */

    public org.osid.resource.ResourceQuery getSponsorQuery();


    /**
     *  Clears the sponsor terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSponsorTerms();


    /**
     *  Matches events whose locations contain the given coordinate. 
     *
     *  @param  coordinate a coordinate 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> coordinate </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchCoordinate(org.osid.mapping.Coordinate coordinate, 
                                boolean match);


    /**
     *  Clears the cooordinate terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCoordinateTerms();


    /**
     *  Matches events whose locations fall within the given spatial unit. 
     *
     *  @param  spatialUnit a spatial unit 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> spatialUnit </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchSpatialUnit(org.osid.mapping.SpatialUnit spatialUnit, 
                                 boolean match);


    /**
     *  Clears the spatial unit terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearSpatialUnitTerms();


    /**
     *  Sets the commitment <code> Id </code> for this query. 
     *
     *  @param  commitmentId a commitment <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> commitmentId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchCommitmentId(org.osid.id.Id commitmentId, boolean match);


    /**
     *  Clears the commitment <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCommitmentIdTerms();


    /**
     *  Tests if a <code> CommitmentQuery </code> is available for querying 
     *  recurring event terms. 
     *
     *  @return <code> true </code> if a commitment query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsCommitmentQuery();


    /**
     *  Gets the query for a commitment. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the commitment query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsCommitmentQuery() </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsCommitmentQuery() </code> is <code> true. </code> 
     */

    public org.osid.calendaring.CommitmentQuery getCommitmentQuery();


    /**
     *  Matches an event that has any commitment. 
     *
     *  @param  match <code> true </code> to match events with any commitment, 
     *          <code> false </code> to match events with no commitments 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyCommitment(boolean match);


    /**
     *  Clears the commitment terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCommitmentTerms();


    /**
     *  Sets the event <code> Id </code> for this query to match events that 
     *  have the specified event as an ancestor. 
     *
     *  @param  eventId an event <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> eventId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchContainingEventId(org.osid.id.Id eventId, boolean match);


    /**
     *  Clears the containing event <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearContainingEventIdTerms();


    /**
     *  Tests if a containing event query is available. 
     *
     *  @return <code> true </code> if a containing event query is available, 
     *          <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsContainingEventQuery();


    /**
     *  Gets the query for a containing event. 
     *
     *  @return the containing event query 
     *  @throws org.osid.UnimplementedException <code> 
     *          supportsContainingEventQuery() </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsContainingEventQuery() </code> is <code> true. 
     *              </code> 
     */

    public org.osid.calendaring.EventQuery getContainingEventQuery();


    /**
     *  Matches events with any ancestor event. 
     *
     *  @param  match <code> true </code> to match events with any ancestor 
     *          event, <code> false </code> to match events with no ancestor 
     *          events 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchAnyContainingEvent(boolean match);


    /**
     *  Clears the containing event terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearContainingEventTerms();


    /**
     *  Sets the calendar <code> Id </code> for this query. 
     *
     *  @param  calendarId a calendar <code> Id </code> 
     *  @param  match <code> true </code> for a positive match, <code> false 
     *          </code> for a negative match 
     *  @throws org.osid.NullArgumentException <code> calendarId </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void matchCalendarId(org.osid.id.Id calendarId, boolean match);


    /**
     *  Clears the calendar <code> Id </code> terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCalendarIdTerms();


    /**
     *  Tests if a <code> CalendarQuery </code> is available for querying 
     *  calendars. 
     *
     *  @return <code> true </code> if a calendar query is available, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsCalendarQuery();


    /**
     *  Gets the query for a calendar. Multiple retrievals produce a nested 
     *  <code> OR </code> term. 
     *
     *  @return the calendar query 
     *  @throws org.osid.UnimplementedException <code> supportsCalendarQuery() 
     *          </code> is <code> false </code> 
     *  @compliance optional This method must be implemented if <code> 
     *              supportsCalendarQuery() </code> is <code> true. </code> 
     */

    public org.osid.calendaring.CalendarQuery getCalendarQuery();


    /**
     *  Clears the calendar terms. 
     *
     *  @compliance mandatory This method must be implemented. 
     */

    public void clearCalendarTerms();


    /**
     *  Gets the event query record corresponding to the given <code> Event 
     *  </code> record <code> Type. </code> Multiple retrievals produce a 
     *  nested <code> OR </code> term. 
     *
     *  @param  eventRecordType an event query record type 
     *  @return the event query record 
     *  @throws org.osid.NullArgumentException <code> eventRecordType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.UnsupportedException <code> 
     *          hasRecordType(eventRecordType) </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.records.EventQueryRecord getEventQueryRecord(org.osid.type.Type eventRecordType)
        throws org.osid.OperationFailedException;
}
