//
// org.osid.contact.ContactLookupSession
//
//     The OSID Java specification for org.osid.contact.ContactLookupSession.
//
// Copyright (C) 2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may nodify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.contact;


/**
 *  <p>This session defines methods for retrieving contacts. </p> 
 *  
 *  <p> This lookup session defines several views: </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete result set or is an error 
 *      condition </li> 
 *      <li> isolated address book view: All contact methods in this session 
 *      operate, retrieve and pertain to contacts defined explicitly in the 
 *      current address book. Using an isolated view is useful for managing 
 *      contacts with the <code> ContactAdminSession. </code> </li> 
 *      <li> federated address book view: All contact lookup methods in this 
 *      session operate, retrieve and pertain to all contacts defined in this 
 *      address book and any other address books implicitly available in this 
 *      address book through address book inheritence. </li> 
 *      <li> effective contact view: All contact lookup methods return 
 *      contacts where the current dates falls in between the effective dates 
 *      inclusive. </li> 
 *      <li> any effective contact view: Contacts of any effective date are 
 *      returned. </li> 
 *  </ul>
 *  
 *  
 *  <p> The methods <code> useFederatedAddressBookView() </code> and <code> 
 *  useIsolatedAddressBookView() </code> behave as a radio group and one 
 *  should be selected before invoking any lookup methods. </p>
 */

public interface ContactLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> AddressBook </code> <code> Id </code> associated with 
     *  this session. 
     *
     *  @return the <code> AddressBook Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getAddressBookId();


    /**
     *  Gets the <code> AddressBook </code> associated with this session. 
     *
     *  @return the address book 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.AddressBook getAddressBook()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can lookup contacts. A return of true does not 
     *  guarantee successful authorization. A return of false indicates that 
     *  it is known all methods in this session will result in a <code> 
     *  PERMISSION_DENIED. </code> This is intended as a hint to an 
     *  application that may opt not to offer these operations. 
     *
     *  @return <code> false </code> if contact lookup methods are not 
     *          authorized, <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupContacts();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeContactView();


    /**
     *  A complete view of the <code> Contact </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryContactView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include contacts in address books which are children of this address 
     *  book in the address book hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedAddressBookView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts retrievals to this address book only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedAddressBookView();


    /**
     *  Only contacts whose effective dates are current are returned by 
     *  methods in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveContactView();


    /**
     *  All contacts of any effective dates are returned by all methods in 
     *  this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveContactView();


    /**
     *  Gets the <code> Contact </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Contact </code> may have a different <code> Id </code> than requested, 
     *  such as the case where a duplicate <code> Id </code> was assigned to a 
     *  <code> Contact </code> and retained for compatibility. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  contactId the <code> Id </code> of the <code> Contact </code> 
     *          to retrieve 
     *  @return the returned <code> Contact </code> 
     *  @throws org.osid.NotFoundException no <code> Contact </code> found 
     *          with the given <code> Id </code> 
     *  @throws org.osid.NullArgumentException <code> contactId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.Contact getContact(org.osid.id.Id contactId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ContactList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  contacts specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible contacts may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  contactIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Contact list </code> 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> contactIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByIds(org.osid.id.IdList contactIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ContactList </code> corresponding to the given contact 
     *  genus <code> Type </code> which does not include contacts of genus 
     *  types derived from the specified <code> Type. </code> 
     *  
     *  In plenary mode, the returned list contains all of the contacts 
     *  specified in the <code> Id </code> list, in the order of the list, 
     *  including duplicates, or an error results if an Id in the supplied 
     *  list is not found or inaccessible. Otherwise, inaccessible contacts 
     *  may be omitted from the list and may present the elements in any order 
     *  including returning a unique set. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  contactGenusType a contact genus type 
     *  @return the returned <code> Contact </code> list 
     *  @throws org.osid.NullArgumentException <code> contactGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusType(org.osid.type.Type contactGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ContactList </code> corresponding to the given contact 
     *  genus <code> Type </code> and include any additional contacts with 
     *  genus types derived from the specified <code> Type. </code> 
     *  
     *  In plenary mode, the returned list contains all of the contacts 
     *  specified in the <code> Id </code> list, in the order of the list, 
     *  including duplicates, or an error results if an Id in the supplied 
     *  list is not found or inaccessible. Otherwise, inaccessible contacts 
     *  may be omitted from the list and may present the elements in any order 
     *  including returning a unique set. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  contactGenusType a contact genus type 
     *  @return the returned <code> Contact </code> list 
     *  @throws org.osid.NullArgumentException <code> contactGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByParentGenusType(org.osid.type.Type contactGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ContactList </code> containing the given contact record 
     *  <code> Type. </code> 
     *  
     *  In plenary mode, the returned list contains all of the contacts 
     *  specified in the <code> Id </code> list, in the order of the list, 
     *  including duplicates, or an error results if an Id in the supplied 
     *  list is not found or inaccessible. Otherwise, inaccessible contacts 
     *  may be omitted from the list and may present the elements in any order 
     *  including returning a unique set. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  contactRecordType a contact record type 
     *  @return the returned <code> Contact </code> list 
     *  @throws org.osid.NullArgumentException <code> contactRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByRecordType(org.osid.type.Type contactRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of contacts with an effective during the entire given date 
     *  range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  from the starting date 
     *  @param  to the ending date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from </code> or <code> 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsOnDate(org.osid.calendaring.DateTime from, 
                                                          org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of contacts of a genus type and with an effective during 
     *  the entire given date range inclusive but not confined to the date 
     *  range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  contactGenusType a contact genus type 
     *  @param  from the starting date 
     *  @param  to the ending date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> contactGenusType, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusTypeOnDate(org.osid.type.Type contactGenusType, 
                                                                     org.osid.calendaring.DateTime from, 
                                                                     org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of contacts corresponding to a reference <code> Id. 
     *  </code> 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId the <code> Id </code> of the reference 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsForReference(org.osid.id.Id referenceId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts corresponding to a reference <code> Id 
     *  </code> and effective during the entire given date range inclusive but 
     *  not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId a reference <code> Id </code> 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId, from 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsForReferenceOnDate(org.osid.id.Id referenceId, 
                                                                      org.osid.calendaring.DateTime from, 
                                                                      org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of contacts of the given genus type corresponding to a 
     *  reference <code> Id. </code> 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId the <code> Id </code> of the reference 
     *  @param  contactGenusType the genus type of the contact 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId </code> or 
     *          <code> contactGenusType </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusTypeForReference(org.osid.id.Id referenceId, 
                                                                           org.osid.type.Type contactGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts of the given genus type corresponding to a 
     *  reference <code> Id </code> and effective during the entire given date 
     *  range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId a reference <code> Id </code> 
     *  @param  contactGenusType the genus type of the contact 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId, 
     *          contactGenusType, from </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusTypeForReferenceOnDate(org.osid.id.Id referenceId, 
                                                                                 org.osid.type.Type contactGenusType, 
                                                                                 org.osid.calendaring.DateTime from, 
                                                                                 org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts corresponding to an address <code> Id. 
     *  </code> 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  addressId the <code> Id </code> of the address 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.NullArgumentException <code> addressId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsForAddress(org.osid.id.Id addressId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts corresponding to an address <code> Id 
     *  </code> and effective during the entire given date range inclusive but 
     *  not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  addressId an address <code> Id </code> 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> addressId, from </code> 
     *          or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsForAddressOnDate(org.osid.id.Id addressId, 
                                                                    org.osid.calendaring.DateTime from, 
                                                                    org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts of the given genus type corresponding to 
     *  an address <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  contacts or an error results. Otherwise, the returned list may contain 
     *  only those contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  addressId the <code> Id </code> of the address 
     *  @param  contactGenusType the genus type of the contact 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.NullArgumentException <code> addressId </code> or 
     *          <code> contactGenusType </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusTypeForAddress(org.osid.id.Id addressId, 
                                                                         org.osid.type.Type contactGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts of the given genus type corresponding to 
     *  an address <code> Id </code> and effective during the entire given 
     *  date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  addressId an address <code> Id </code> 
     *  @param  contactGenusType the genus type of the contact 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> addressId, 
     *          contactGenusType, from </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusTypeForAddressOnDate(org.osid.id.Id addressId, 
                                                                               org.osid.type.Type contactGenusType, 
                                                                               org.osid.calendaring.DateTime from, 
                                                                               org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts corresponding to a reference and address 
     *  <code> Id. </code> 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId the <code> Id </code> of the reference 
     *  @param  addressId the <code> Id </code> of the address 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId </code> or 
     *          <code> addressId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsForReferenceAndAddress(org.osid.id.Id referenceId, 
                                                                          org.osid.id.Id addressId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts corresponding to a reference and address 
     *  <code> Id </code> and effectiveduring the entire given date range 
     *  inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId the <code> Id </code> of the reference 
     *  @param  addressId an address <code> Id </code> 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId, addressI, 
     *          from </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsForReferenceAndAddressOnDate(org.osid.id.Id referenceId, 
                                                                                org.osid.id.Id addressId, 
                                                                                org.osid.calendaring.DateTime from, 
                                                                                org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts with the given genus type corresponding to 
     *  a reference and address <code> Id. </code> 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId the <code> Id </code> of the reference 
     *  @param  addressId the <code> Id </code> of the address 
     *  @param  contactGenusType the genus type of the contact 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId, addressId 
     *          </code> or <code> contactGenusType </code> is <code> null 
     *          </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusTypeForReferenceAndAddress(org.osid.id.Id referenceId, 
                                                                                     org.osid.id.Id addressId, 
                                                                                     org.osid.type.Type contactGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all contacts with the given genus type corresponding to 
     *  a reference and address <code> Id </code> and during the entire given 
     *  date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @param  referenceId the <code> Id </code> of the reference 
     *  @param  addressId an address <code> Id </code> 
     *  @param  contactGenusType the genus type of the contact 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> ContactList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> referenceId, addressId, 
     *          contactGenusType, from </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContactsByGenusTypeForReferenceAndAddressOnDate(org.osid.id.Id referenceId, 
                                                                                           org.osid.id.Id addressId, 
                                                                                           org.osid.type.Type contactGenusType, 
                                                                                           org.osid.calendaring.DateTime from, 
                                                                                           org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all contacts. 
     *  
     *  In plenary mode, the returned list contains all known contacts or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  contacts that are accessible through this session. 
     *  
     *  In effective mode, contacts are returned that are currently effective. 
     *  In any effective mode, effective contacts and those currently expired 
     *  are returned. 
     *
     *  @return a list of contacts 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.contact.ContactList getContacts()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
