//
// org.osid.installation.PackageLookupSession
//
//     The OSID Java specification for org.osid.installation.PackageLookupSession.
//
// Copyright (C) 2006 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2009-2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.installation;


/**
 *  <p>This session provides methods for retrieving <code> Package </code> s 
 *  from remote <code> Depots. </code> </p> 
 *  
 *  <p> This session defines views that offer differing behaviors when 
 *  retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete set or is an error condition 
 *      </li> 
 *      <li> isolated depot view: All package methods in this session operate, 
 *      retrieve and pertain to packages defined explicitly in the current 
 *      depot. Using an isolated view is useful for managing packages with the 
 *      <code> PackageAdminSession. </code> </li> 
 *      <li> federated depot view: All package methods in this session 
 *      operate, retrieve and pertain to all packages defined in this depot 
 *      and any other depots implicitly available in this depot through depot 
 *      inheritence. </li> 
 *      <li> normalized version view: multiple versions of the same package 
 *      are suppressed </li> 
 *      <li> denormalized version vew: all versions of an installation are 
 *      returned </li> 
 *  </ul>
 *  
 *  
 *  <p> Packages may have an additional records indicated by their respective 
 *  record types. The record may not be accessed through a cast of the <code> 
 *  Package. </code> </p>
 */

public interface PackageLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Depot </code> <code> Id </code> associated with this 
     *  session. 
     *
     *  @return the <code> Depot Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getDepotId();


    /**
     *  Gets the <code> Depot </code> associated with this session. 
     *
     *  @return the <code> Depot </code> associated with this session 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.Depot getDepot()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Package </code> lookups. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may opt not to offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupPackages();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativePackageView();


    /**
     *  A complete view of the <code> Package </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryPackageView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include packages in depots which are children of this depot in the 
     *  depot hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedDepotView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this depot only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedDepotView();


    /**
     *  The returns from the lookup methods may omit multiple versions of the 
     *  same package. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useNormalizedVersionView();


    /**
     *  All versions of the same package are returned. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useDenormalizedVersionView();


    /**
     *  Gets the <code> Package </code> specified by its <code> Id. </code> In 
     *  plenary mode, the exact <code> Id </code> is found or a <code> 
     *  NOT_FOUND </code> results. Otherwise, the returned <code> Package 
     *  </code> may have a different <code> Id </code> than requested, such as 
     *  the case where a duplicate <code> Id </code> was assigned to a <code> 
     *  Package </code> and retained for compatibility. 
     *
     *  @param  packageId <code> Id </code> of the <code> Package </code> 
     *  @return the package 
     *  @throws org.osid.NotFoundException <code> packageId </code> not found 
     *  @throws org.osid.NullArgumentException <code> packageId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method is must be implemented. 
     */

    public org.osid.installation.Package getPackage(org.osid.id.Id packageId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> PackageList </code> corresponding to the given <code> 
     *  IdList. </code> In plenary mode, the returned list contains all of the 
     *  packages specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Packages </code> may be omitted from the list and 
     *  may present the elements in any order including returning a unique 
     *  set. 
     *
     *  @param  packageIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Package </code> list 
     *  @throws org.osid.NotFoundException an <code> Id </code> was not found 
     *  @throws org.osid.NullArgumentException <code> packageIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackagesByIds(org.osid.id.IdList packageIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> PackageList </code> corresponding to the given package 
     *  genus <code> Type </code> which does not include packages of genus 
     *  types derived from the specified <code> Type. </code> In plenary mode, 
     *  the returned list contains all known packages or an error results. 
     *  Otherwise, the returned list may contain only those packages that are 
     *  accessible through this session. 
     *
     *  @param  packageGenusType a package genus type 
     *  @return the returned <code> Package </code> list 
     *  @throws org.osid.NullArgumentException <code> packageGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackagesByGenusType(org.osid.type.Type packageGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> PackageList </code> corresponding to the given package 
     *  genus <code> Type </code> and include any additional package with 
     *  genus types derived from the specified <code> Type. </code> In plenary 
     *  mode, the returned list contains all known packages or an error 
     *  results. Otherwise, the returned list may contain only those packages 
     *  that are accessible through this session. 
     *
     *  @param  packageGenusType a package genus type 
     *  @return the returned <code> Package </code> list 
     *  @throws org.osid.NullArgumentException <code> packageGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackagesByParentGenusType(org.osid.type.Type packageGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> PackageList </code> containing the given package record 
     *  <code> Type. </code> In plenary mode, the returned list contains all 
     *  known packages or an error results. Otherwise, the returned list may 
     *  contain only those packages that are accessible through this session. 
     *
     *  @param  packageRecordType a package record type 
     *  @return the returned <code> Package </code> list 
     *  @throws org.osid.NullArgumentException <code> packageRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackagesByRecordType(org.osid.type.Type packageRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> PackageList </code> for the given provider. In plenary 
     *  mode, the returned list contains all known packages or an error 
     *  results. Otherwise, the returned list may contain only those packages 
     *  that are accessible through this session. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @return the returned <code> Package </code> list 
     *  @throws org.osid.NullArgumentException <code> resourceId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackagesByProvider(org.osid.id.Id resourceId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of packages depending on the given package. 
     *
     *  @param  packageId an <code> Id </code> of a <code> Package </code> 
     *  @return list of package dependents 
     *  @throws org.osid.NotFoundException <code> packageId </code> is not 
     *          found 
     *  @throws org.osid.NullArgumentException <code> packageId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getDependentPackages(org.osid.id.Id packageId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of packages in the specified package version chain. 
     *
     *  @param  packageId an <code> Id </code> of a <code> Package </code> 
     *  @return list of dependencies 
     *  @throws org.osid.NotFoundException <code> packageId </code> is not 
     *          found 
     *  @throws org.osid.NullArgumentException <code> packageId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackageVersions(org.osid.id.Id packageId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Packages. </code> In plenary mode, the returned list 
     *  contains all known packages or an error results. Otherwise, the 
     *  returned list may contain only those packages that are accessible 
     *  through this session. 
     *
     *  @return a <code> PackageList </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackages()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
