//
// org.osid.installation.PackageQuerySession
//
//     The OSID Java specification for org.osid.installation.PackageQuerySession.
//
// Copyright (C) 2006 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2009-2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.installation;


/**
 *  <p>This session provides methods for searching <code> Package </code> 
 *  objects. The search query is constructed using the <code> PackageQuery. 
 *  </code> The package record <code> Type </code> also specifies the record 
 *  for the package query. </p> 
 *  
 *  <p> This session defines views that offer differing behaviors for 
 *  searching. </p> 
 *  
 *  <ul>
 *      <li> federated depot view: searches include packages in depots of 
 *      which this depot is a ancestor in the depot hierarchy </li> 
 *      <li> isolated depot view: searches are restricted to packages in this 
 *      depot </li> 
 *      <li> normalized version view: multiple versions of the same package 
 *      are suppressed </li> 
 *      <li> denormalized version vew: all versions of an installation are 
 *      returned </li> 
 *      <li> normalized dependency view: supporting installations upon which 
 *      other installations depend are suppressed </li> 
 *      <li> denormalized dependency view: all dependencies are returned </li> 
 *  </ul>
 *  
 *  
 *  <p> Packages may have a query record indicated by their respective record 
 *  types. The query record is accessed via the <code> PackageQuery. </code> 
 *  </p>
 */

public interface PackageQuerySession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Depot </code> <code> Id </code> associated with this 
     *  session. 
     *
     *  @return the <code> Depot Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getDepotId();


    /**
     *  Gets the <code> Depot </code> associated with this session. 
     *
     *  @return the <code> Depot </code> associated with this session 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.Depot getDepot()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Package </code> searches. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may opt not to offer search operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if search methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canSearchPackages();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include packages in depots which are children of this depot in the 
     *  depot hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedDepotView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts searches to this depot only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedDepotView();


    /**
     *  The returns from the lookup methods may omit multiple versions of the 
     *  same package. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useNormalizedVersionView();


    /**
     *  All versions of the same package are returned. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useDenormalizedVersionView();


    /**
     *  A normalized view uses a single <code> Package </code> to represent a 
     *  set of package dependencies. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useNormalizedDependencyView();


    /**
     *  A denormalized view returns all dependencies. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useDenormalizedDependencyView();


    /**
     *  Gets a package query. 
     *
     *  @return the package query 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageQuery getPackageQuery();


    /**
     *  Gets a list of <code> Packages </code> matching the given package 
     *  query. 
     *
     *  @param  packageQuery the package query 
     *  @return the returned <code> PackageList </code> 
     *  @throws org.osid.NullArgumentException <code> packageQuery </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @throws org.osid.UnsupportedException <code> packageQuery </code> is 
     *          not of this service 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.PackageList getPackagesByQuery(org.osid.installation.PackageQuery packageQuery)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
