//
// org.osid.learning.ObjectiveRequisiteSession
//
//     The OSID Java specification for org.osid.learning.ObjectiveRequisiteSession.
//
// Copyright (C) 2009 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2011 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may nodify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.learning;


/**
 *  <p>This session provides methods for retrieving objective requisites. A 
 *  requisite is a set of <code> Objectives </code> that should be achieved 
 *  before another <code> Objective </code> is attempted. </p> 
 *  
 *  <p> This session defines views that offer differing behaviors when 
 *  retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete set or is an error condition 
 *      </li> 
 *      <li> isolated objective bank view: All objective methods in this 
 *      session operate, retrieve and pertain to objectives defined explicitly 
 *      in the current objective bank. Using an isolated view is useful for 
 *      managing objectives with the <code> ObjectiveAdminSession. </code> 
 *      </li> 
 *      <li> federated objective bank view: All objective methods in this 
 *      session operate, retrieve and pertain to all objectives defined in 
 *      this objective bank and any other objective banks implicitly available 
 *      in this objective bank through objective bank inheritence. </li> 
 *  </ul>
 *  
 *  
 *  <p> Objectives may have an additional records indicated by their 
 *  respective record types. The record may not be accessed through a cast of 
 *  the <code> Objective. </code> </p>
 */

public interface ObjectiveRequisiteSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> ObjectiveBank </code> <code> Id </code> associated 
     *  with this session. 
     *
     *  @return the <code> ObjectiveBank Id </code> associated with this 
     *          session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getObjectiveBankId();


    /**
     *  Gets the <code> ObjectiveBank </code> associated with this session. 
     *
     *  @return the <code> ObjectiveBank </code> associated with this session 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.learning.ObjectiveBank getObjectiveBank()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Objective </code> lookups. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may opt not to offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupObjectivePrerequisites();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeObjectiveView();


    /**
     *  A complete view of the <code> Objective </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryObjectiveView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include objectives in objective banks which are children of this 
     *  objective bank in the objective bank hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedObjectiveBankView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this objective bank only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedObjectiveBankView();


    /**
     *  Gets a list of <code> Objectives </code> that are the immediate 
     *  requisites for the given <code> Objective. </code> In plenary mode, 
     *  the returned list contains all of the immediate requisites, or an 
     *  error results if an <code> Objective </code> is not found or 
     *  inaccessible. Otherwise, inaccessible <code> Objectives </code> may be 
     *  omitted from the list and may present the elements in any order 
     *  including returning a unique set. 
     *
     *  @param  objectiveId <code> Id </code> of the <code> Objective </code> 
     *  @return the returned requisite <code> Objectives </code> 
     *  @throws org.osid.NotFoundException <code> objectiveId </code> not 
     *          found 
     *  @throws org.osid.NullArgumentException <code> objectiveId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method is must be implemented. 
     */

    public org.osid.learning.ObjectiveList getRequisiteObjectives(org.osid.id.Id objectiveId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of <code> Objectives </code> that are the requisites for 
     *  the given <code> Objective </code> including the requistes of the 
     *  requisites, and so on. In plenary mode, the returned list contains all 
     *  of the immediate requisites, or an error results if an <code> 
     *  Objective </code> is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Objectives </code> may be omitted from the list 
     *  and may present the elements in any order including returning a unique 
     *  set. 
     *
     *  @param  objectiveId <code> Id </code> of the <code> Objective </code> 
     *  @return the returned <code> Objective </code> list 
     *  @throws org.osid.NotFoundException <code> objectiveId </code> not 
     *          found 
     *  @throws org.osid.NullArgumentException <code> objectiveId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.learning.ObjectiveList getAllRequisiteObjectives(org.osid.id.Id objectiveId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of <code> Objectives </code> that require the given <code> 
     *  Objective. </code> In plenary mode, the returned list contains all of 
     *  the immediate requisites, or an error results if an Objective is not 
     *  found or inaccessible. Otherwise, inaccessible <code> Objectives 
     *  </code> may be omitted from the list and may present the elements in 
     *  any order including returning a unique set. 
     *
     *  @param  objectiveId <code> Id </code> of the <code> Objective </code> 
     *  @return the returned <code> Objective </code> list 
     *  @throws org.osid.NotFoundException <code> objectiveId </code> not 
     *          found 
     *  @throws org.osid.NullArgumentException <code> objectiveId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.learning.ObjectiveList getDependentObjectives(org.osid.id.Id objectiveId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if an objective is required before proceeding with an objective. 
     *  One objective may indirectly depend on another objective by way of one 
     *  or more other objectives. 
     *
     *  @param  objectiveId <code> Id </code> of the dependent <code> 
     *          Objective </code> 
     *  @param  requiredObjectiveId <code> Id </code> of the required <code> 
     *          Objective </code> 
     *  @return <code> true </code> if <code> objectiveId </code> depends on 
     *          <code> requiredObjectiveId, </code> <code> false </code> 
     *          otherwise 
     *  @throws org.osid.NotFoundException <code> objectiveId </code> not 
     *          found 
     *  @throws org.osid.NullArgumentException <code> objectiveId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isObjectiveRequired(org.osid.id.Id objectiveId, 
                                       org.osid.id.Id requiredObjectiveId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of <code> Objectives </code> that are equivalent to the 
     *  given <code> Objective </code> for the purpose of requisites. An 
     *  equivalent objective can satisfy the given objective. In plenary mode, 
     *  the returned list contains all of the equivalent requisites, or an 
     *  error results if an Objective is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Objectives </code> may be omitted from the list 
     *  and may present the elements in any order including returning a unique 
     *  set. 
     *
     *  @param  objectiveId <code> Id </code> of the <code> Objective </code> 
     *  @return the returned <code> Objective </code> list 
     *  @throws org.osid.NotFoundException <code> objectiveId </code> not 
     *          found 
     *  @throws org.osid.NullArgumentException <code> objectiveId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.learning.ObjectiveList getEquivalentObjectives(org.osid.id.Id objectiveId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
