//
// org.osid.provisioning.RequestLookupSession
//
//     The OSID Java specification for org.osid.provisioning.RequestLookupSession.
//
// Copyright (C) 2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may nodify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.provisioning;


/**
 *  <p>This session provides methods for retrieving <code> Request </code> 
 *  relationships. </p> 
 *  
 *  <p> This session defines views that offer differing behaviors when 
 *  retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete set or is an error condition 
 *      </li> 
 *      <li> isolated distributor view: All request methods in this session 
 *      operate, retrieve and pertain to requests defined explicitly in the 
 *      current distributor. Using an isolated view is useful for managing 
 *      <code> Requests </code> with the <code> RequestAdminSession. </code> 
 *      </li> 
 *      <li> federated distributor view: All request methods in this session 
 *      operate, retrieve and pertain to all requests defined in this 
 *      distributor and any other requests implicitly available in this 
 *      distributor through distributor inheritence. </li> 
 *      <li> effective request view: All request lookup methods return 
 *      requests where the current date falls in between the effective dates 
 *      inclusive. </li> 
 *      <li> any effective request view: Requests of any effective date are 
 *      returned from methods. </li> 
 *  </ul>
 *  
 *  
 *  <p> Generally, the comparative view should be used for most applications 
 *  as it permits operation even if there is data that cannot be accessed. The 
 *  methods <code> useFederatedDistributorView() </code> and <code> 
 *  useIsolatedDistributorView() </code> behave as a radio group and one 
 *  should be selected before invoking any lookup methods. </p>
 */

public interface RequestLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Distributor </code> <code> Id </code> associated with 
     *  this session. 
     *
     *  @return the <code> Distributor Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getDistributorId();


    /**
     *  Gets the <code> Distributor </code> associated with this session. 
     *
     *  @return the distributor 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.Distributor getDistributor()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Request </code> lookups. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may opt not to offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupRequests();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeRequestView();


    /**
     *  A complete view of the <code> Request </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryRequestView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include requests in distributors which are children of this 
     *  distributor in the distributor hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedDistributorView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this distributor only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedDistributorView();


    /**
     *  Only requests whose effective dates are current are returned by 
     *  methods in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveRequestView();


    /**
     *  All requests of any effective dates are returned by methods in this 
     *  session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveRequestView();


    /**
     *  Gets the <code> Request </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Request </code> may have a different <code> Id </code> than requested, 
     *  such as the case where a duplicate <code> Id </code> was assigned to a 
     *  <code> Request </code> and retained for compatibility. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  requestId <code> Id </code> of the <code> Request </code> 
     *  @return the request 
     *  @throws org.osid.NotFoundException <code> requestId </code> not found 
     *  @throws org.osid.NullArgumentException <code> requestId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method is must be implemented. 
     */

    public org.osid.provisioning.Request getRequest(org.osid.id.Id requestId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RequestList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  requests specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Requests </code> may be omitted from the list and 
     *  may present the elements in any order including returning a unique 
     *  set. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  requestIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.NotFoundException an <code> Id </code> was not found 
     *  @throws org.osid.NullArgumentException <code> requestIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsByIds(org.osid.id.IdList requestIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RequestList </code> corresponding to the given request 
     *  genus <code> Type </code> which does not include requests of genus 
     *  types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  requestGenusType a request genus type 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.NullArgumentException <code> requestGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsByGenusType(org.osid.type.Type requestGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RequestList </code> corresponding to the given request 
     *  genus <code> Type </code> and include any additional requests with 
     *  genus types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  requestGenusType a request genus type 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.NullArgumentException <code> requestGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsByParentGenusType(org.osid.type.Type requestGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RequestList </code> containing the given request record 
     *  <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  requestRecordType a request record type 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.NullArgumentException <code> requestRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsByRecordType(org.osid.type.Type requestRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of requests effective during the entire given date range 
     *  inclusive but not confined to the date range. 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective 
     *  in addition to being effective during the given date range. In any 
     *  effective mode, effective requests and those currently expired are 
     *  returned. 
     *
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from </code> or <code> 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsOnDate(org.osid.calendaring.DateTime from, 
                                                               org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of requests for a requester. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.NullArgumentException <code> resourceId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsForRequester(org.osid.id.Id resourceId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of requests for a requester and effective during the 
     *  entire given date range inclusive but not confined to the date range. 
     *  <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective 
     *  in addition to being effective during the given date range. In any 
     *  effective mode, effective requests and those currently expired are 
     *  returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> resourceId, from </code> 
     *          or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsForRequesterOnDate(org.osid.id.Id resourceId, 
                                                                           org.osid.calendaring.DateTime from, 
                                                                           org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of requests for a queue. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  queueId a queue <code> Id </code> 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.NullArgumentException <code> queueId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsForQueue(org.osid.id.Id queueId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of requests for a queue and effective during the entire 
     *  given date range inclusive but not confined to the date range. <code> 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective 
     *  in addition to being effective during the given date range. In any 
     *  effective mode, effective requests and those currently expired are 
     *  returned. 
     *
     *  @param  queueId a queue <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> queueId, from </code> or 
     *          <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsForQueueOnDate(org.osid.id.Id queueId, 
                                                                       org.osid.calendaring.DateTime from, 
                                                                       org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of requests for a requester and queue. 
     *  
     *  In plenary mode, the returned list contains all known requests or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @param  queueId a queue <code> Id </code> 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.NullArgumentException <code> resourceId </code> or 
     *          <code> queueId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsForRequesterAndQueue(org.osid.id.Id resourceId, 
                                                                             org.osid.id.Id queueId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of requests for a requester and queue and effective during 
     *  the entire given date range inclusive but not confined to the date 
     *  range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  requests or an error results. Otherwise, the returned list may contain 
     *  only those requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective 
     *  in addition to being effective during the given date range. In any 
     *  effective mode, effective requests and those currently expired are 
     *  returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @param  queueId a queue <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Request </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> resourceId, queueId, 
     *          from </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequestsForRequesterAndQueueOnDate(org.osid.id.Id resourceId, 
                                                                                   org.osid.id.Id queueId, 
                                                                                   org.osid.calendaring.DateTime from, 
                                                                                   org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Requests. </code> 
     *  
     *  In plenary mode, the returned list contains all known requests or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  requests that are accessible through this session. 
     *  
     *  In effective mode, requests are returned that are currently effective. 
     *  In any effective mode, effective requests and those currently expired 
     *  are returned. 
     *
     *  @return a list of <code> Requests </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.provisioning.RequestList getRequests()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
