//
// org.osid.room.RoomLookupSession
//
//     The OSID Java specification for org.osid.room.RoomLookupSession.
//
// Copyright (C) 2011 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.room;


/**
 *  <p>This session defines methods for retrieving rooms. </p> 
 *  
 *  <p> This lookup session defines several views: </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete result set or is an error 
 *      condition </li> 
 *      <li> isolated campus view: All room methods in this session operate, 
 *      retrieve and pertain to rooms defined explicitly in the current 
 *      campus. Using an isolated view is useful for managing rooms with the 
 *      <code> RoomAdminSession. </code> </li> 
 *      <li> federated campus view: All room lookup methods in this session 
 *      operate, retrieve and pertain to all rooms defined in this campus and 
 *      any other campuses implicitly available in this campus through campus 
 *      inheritence. </li> 
 *      <li> effective room view: All room lookup methods return effective 
 *      rooms. </li> 
 *      <li> any effective room view: Rooms of any effective date are 
 *      returned. </li> 
 *  </ul>
 *  
 *  
 *  <p> The methods <code> useFederatedCampusView() </code> and <code> 
 *  useIsolatedCampusView() </code> behave as a radio group and one should be 
 *  selected before invoking any lookup methods. </p>
 */

public interface RoomLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Campus </code> <code> Id </code> associated with this 
     *  session. 
     *
     *  @return the <code> Campus Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getCampusId();


    /**
     *  Gets the <code> Campus </code> associated with this session. 
     *
     *  @return the campus 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.Campus getCampus()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can lookup rooms. A return of true does not 
     *  guarantee successful authorization. A return of false indicates that 
     *  it is known all methods in this session will result in a <code> 
     *  PERMISSION_DENIED. </code> This is intended as a hint to an 
     *  application that may opt not to offer these operations. 
     *
     *  @return <code> false </code> if room lookup methods are not 
     *          authorized, <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupRooms();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeRoomView();


    /**
     *  A complete view of the <code> Room </code> returns is desired. Methods 
     *  will return what is requested or result in an error. This view is used 
     *  when greater precision is desired at the expense of interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryRoomView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include rooms in campuses which are children of this campus in the 
     *  campus hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedCampusView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts retrievals to this campus only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedCampusView();


    /**
     *  Only rooms whose effective dates are current are returned by methods 
     *  in this session 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveRoomView();


    /**
     *  Rooms of any effective dates are returned by all methods in this 
     *  session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveRoomView();


    /**
     *  Gets the <code> Room </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Room </code> may have a different <code> Id </code> than requested, 
     *  such as the case where a duplicate <code> Id </code> was assigned to a 
     *  <code> Room </code> and retained for compatibility. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  roomId the <code> Id </code> of the <code> Room </code> to 
     *          retrieve 
     *  @return the returned <code> Room </code> 
     *  @throws org.osid.NotFoundException no <code> Room </code> found with 
     *          the given <code> Id </code> 
     *  @throws org.osid.NullArgumentException <code> roomId </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.Room getRoom(org.osid.id.Id roomId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RoomList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  rooms specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible rooms may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  roomIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Room list </code> 
     *  @throws org.osid.NotFoundException an <code> Id </code> was not found 
     *  @throws org.osid.NullArgumentException <code> roomIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByIds(org.osid.id.IdList roomIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RoomList </code> corresponding to the given room genus 
     *  <code> Type </code> which does not include rooms of genus types 
     *  derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  rooms or an error results. Otherwise, the returned list may contain 
     *  only those rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  roomGenusType a room genus type 
     *  @return the returned <code> Room </code> list 
     *  @throws org.osid.NullArgumentException <code> roomGenusType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByGenusType(org.osid.type.Type roomGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RoomList </code> corresponding to the given room genus 
     *  <code> Type </code> and include any additional rooms with genus types 
     *  derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  rooms or an error results. Otherwise, the returned list may contain 
     *  only those rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  roomGenusType a room genus type 
     *  @return the returned <code> Rooms </code> list 
     *  @throws org.osid.NullArgumentException <code> roomGenusType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByParentGenusType(org.osid.type.Type roomGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> RoomList </code> containing the given room record <code> 
     *  Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  rooms or an error results. Otherwise, the returned list may contain 
     *  only those rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  roomRecordType a room record type 
     *  @return the returned <code> Rooms </code> list 
     *  @throws org.osid.NullArgumentException <code> roomRecordType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByRecordType(org.osid.type.Type roomRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms effective during the entire given date range 
     *  inclusive but not confined to the date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  rooms or an error results. Otherwise, the returned list may contain 
     *  only those rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from </code> or <code> 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsOnDate(org.osid.calendaring.DateTime from, 
                                                 org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms of a genus type effective during the entire 
     *  given date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  roomGenusType a room genus <code> Type </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> roomGenusType, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByGenusTypeOnDate(org.osid.type.Type roomGenusType, 
                                                            org.osid.calendaring.DateTime from, 
                                                            org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms corresponding to a building <code> Id. 
     *  </code> 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  buildingId the <code> Id </code> of the building 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.NullArgumentException <code> buildingId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsForBuilding(org.osid.id.Id buildingId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms corresponding to a building <code> Id </code> 
     *  and effective during the entire given date range inclusive but not 
     *  confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  buildingId a building <code> Id </code> 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> buildingId, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsForBuildingOnDate(org.osid.id.Id buildingId, 
                                                            org.osid.calendaring.DateTime from, 
                                                            org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms of the given genus type corresponding to a 
     *  building <code> Id. </code> 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  buildingId the <code> Id </code> of the building 
     *  @param  roomGenusType a room genus type 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.NullArgumentException <code> buildingId </code> or 
     *          <code> roomGenusType </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByGenusTypeForBuilding(org.osid.id.Id buildingId, 
                                                                 org.osid.type.Type roomGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms of the given genus type corresponding to a 
     *  building <code> Id </code> and effective during the entire given date 
     *  range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  buildingId a building <code> Id </code> 
     *  @param  roomGenusType a room genus type 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> buildingId, 
     *          roomGenusType, from, </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByGenusTypeForBuildingOnDate(org.osid.id.Id buildingId, 
                                                                       org.osid.type.Type roomGenusType, 
                                                                       org.osid.calendaring.DateTime from, 
                                                                       org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms corresponding to a floor <code> Id. </code> 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  floorId the <code> Id </code> of the building 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.NullArgumentException <code> floorId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsForFloor(org.osid.id.Id floorId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms corresponding to a floor <code> Id </code> 
     *  and effective during the entire given date range inclusive but not 
     *  confined to the date range. 
     *  
     *  Rooms are returned with start effective dates that fall between the 
     *  requested dates inclusive. In plenary mode, the returned list contains 
     *  all known rooms or an error results. Otherwise, the returned list may 
     *  contain only those rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  floorId a building <code> Id </code> 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> floorId, from, </code> 
     *          or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsForFloorOnDate(org.osid.id.Id floorId, 
                                                         org.osid.calendaring.DateTime from, 
                                                         org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms of the given genus type corresponding to a 
     *  floor <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  rooms or an error results. Otherwise, the returned list may contain 
     *  only those rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  floorId the <code> Id </code> of the floor 
     *  @param  roomGenusType a room genus type 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.NullArgumentException <code> floorId </code> or 
     *          <code> roomGenusType </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByGenusTypeForFloor(org.osid.id.Id floorId, 
                                                              org.osid.type.Type roomGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms of the given genus type corresponding to a 
     *  floor <code> Id </code> and effective during the entire given date 
     *  range inclusive but not confined to the date range. 
     *  
     *  Rooms are returned with start effective dates that fall between the 
     *  requested dates inclusive. In plenary mode, the returned list contains 
     *  all known rooms or an error results. Otherwise, the returned list may 
     *  contain only those rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  floorId a floor <code> Id </code> 
     *  @param  roomGenusType a room genus type 
     *  @param  from from date 
     *  @param  to to date 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.InvalidArgumentException <code> to </code> is less 
     *          than <code> from </code> 
     *  @throws org.osid.NullArgumentException <code> floorId, roomGenusType, 
     *          from, </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByGenusTypeForFloorOnDate(org.osid.id.Id floorId, 
                                                                    org.osid.type.Type roomGenusType, 
                                                                    org.osid.calendaring.DateTime from, 
                                                                    org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of all rooms of the given room number. 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @param  number a room number 
     *  @return the returned <code> RoomList </code> 
     *  @throws org.osid.NullArgumentException <code> number </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRoomsByRoomNumber(String number)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all rooms. 
     *  
     *  In plenary mode, the returned list contains all known rooms or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  rooms that are accessible through this session. 
     *  
     *  In effective mode, rooms are returned that are currently effective. In 
     *  any effective mode, effective rooms and those currently expired are 
     *  returned. 
     *
     *  @return a list of rooms 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.room.RoomList getRooms()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
